import { Module } from '@nestjs/common';
import { ProjectsService } from './projects.service';
import { ProjectsResolver } from './projects.resolver';
import { TypeOrmModule } from '@nestjs/typeorm';
import { Project } from './project.entity';
import { RabbitMQModule } from '@golevelup/nestjs-rabbitmq';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { EXCHANGE_PROJECT_FANOUT } from './projects.constants';
import { CommonsModule } from '../commons/commons.module';

@Module({
  imports: [
    CommonsModule,
    TypeOrmModule.forFeature([Project]),
    RabbitMQModule.forRootAsync(RabbitMQModule, {
      imports: [ConfigModule],
      useFactory: (configService: ConfigService) => ({
        uri: configService.get<string>('db.rabbitmq.uri'),
        exchanges: [
          {
            name: EXCHANGE_PROJECT_FANOUT,
            type: 'fanout',
            options: {
              durable: false,
              autoDelete: true,
            },
          },
        ],
      }),
      inject: [ConfigService],
    }),
  ],
  providers: [ProjectsService, ProjectsResolver],
  exports: [ProjectsService],
})
export class ProjectsModule {}
